;;; navi2ch-list.el --- board list module for navi2ch

;; Copyright (C) 2000 by 2$B$A$c$s$M$k(B

;; Author: (not 1)
;; Keywords: network, 2ch

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; http://salad.2ch.net/bbstable.html $B$+$i!":n$C$?J}$,$$$$$s$+$J!#(B

;;; Code:

(require 'navi2ch-board)
(require 'cl)

(defvar navi2ch-list-buffer-name "*navi2ch list*")
(defvar navi2ch-list-window-width 20)
(defvar navi2ch-list-bookmark-mode nil)
(defvar navi2ch-list-category-list nil)
(defvar navi2ch-list-bbstable-url "http://www.2ch.net/bbstable.html")
(defvar navi2ch-list-open-category nil
  "$B:G=i$+$iA4$F$N%+%F%4%j$r3+$/$+$I$&$+(B")

(defvar navi2ch-list-ignore-category-list
  '("$B%A%c%C%H(B" "$B$*3($+$-(B" "$B1?1D(B" "$B%D!<%kN`(B" "$BB>$N7G<(HD(B" "$B$^$A#B#B#S(B"))

(defvar navi2ch-list-mode-map nil)
(unless navi2ch-list-mode-map
  (setq navi2ch-list-mode-map (make-sparse-keymap))
  (define-key navi2ch-list-mode-map "\r" 'navi2ch-list-select-current-board)
  (define-key navi2ch-list-mode-map "q" 'navi2ch-exit)
  (define-key navi2ch-list-mode-map "z" 'navi2ch-suspend)
  (define-key navi2ch-list-mode-map "s" 'navi2ch-list-sync)
  (define-key navi2ch-list-mode-map " " 'navi2ch-list-select-current-board)
  (define-key navi2ch-list-mode-map [del] 'scroll-down)
  (define-key navi2ch-list-mode-map [backspace] 'scroll-down)
  (define-key navi2ch-list-mode-map "n" 'next-line)
  (define-key navi2ch-list-mode-map "p" 'previous-line)
  (define-key navi2ch-list-mode-map "a" 'navi2ch-list-add-bookmark)
  (define-key navi2ch-list-mode-map "d" 'navi2ch-list-delete-bookmark)
  (define-key navi2ch-list-mode-map "b" 'navi2ch-list-toggle-bookmark)
  (define-key navi2ch-list-mode-map "t" 'navi2ch-toggle-offline)
  (define-key navi2ch-list-mode-map [mouse-2] 'navi2ch-list-mouse-select)
  (define-key navi2ch-list-mode-map "g" 'navi2ch-list-goto-board)
  (define-key navi2ch-list-mode-map "/" 'navi2ch-list-toggle-open)
  (define-key navi2ch-list-mode-map ">" 'end-of-buffer)
  (define-key navi2ch-list-mode-map "<" 'beginning-of-buffer)
  (define-key navi2ch-list-mode-map "1" 'navi2ch-one-pain)
  (define-key navi2ch-list-mode-map "2" 'navi2ch-list-two-pain)
  (define-key navi2ch-list-mode-map "3" 'navi2ch-three-pain)
  (define-key navi2ch-list-mode-map "\C-c\C-b" 'navi2ch-bookmark))

(defsubst navi2ch-list-get-file-name (&optional name)
  (navi2ch-expand-file-name
   (or name "board.txt")))

(defun navi2ch-list-get-category-list (file)
  (when (file-exists-p file)
    (with-temp-buffer
      (navi2ch-insert-file-contents file)
      (goto-char (point-min))
      (let (alist alist2 name)
        (while (not (eobp))
          (let ((strs (split-string
                       (buffer-substring
                        (point)
                        (and (forward-line 3) (point))) "\n")))
            (if (string= (nth 1 strs) "")
                (progn
                  (when alist2
                    (setq alist (cons (cons name
                                            (list
                                             (cons 'open
                                                   navi2ch-list-open-category)
                                             (cons 'child (nreverse alist2))))
                                      alist)))
                  (setq name (nth 0 strs)
                        alist2 nil))
              (setq alist2 (cons (list (cons 'name (nth 0 strs))
                                       (cons 'uri (nth 1 strs))
                                       (cons 'id (nth 2 strs)))
                                 alist2)))))
        (nreverse alist)))))
    
(defun navi2ch-list-insert-board-names-subr (list)
  (let ((prev (point)))
    (dolist (elt list)
      (insert "  " (cdr (assq 'name elt)) "\n")
      (put-text-property prev (point) 'board elt)
      (put-text-property prev (1- (point)) 'mouse-face 'highlight)
      (setq prev (point)))))

(defun navi2ch-list-insert-board-names (list)
  "`list' $B$NFbMF$r%P%C%U%!$KA^F~(B"
  (let ((prev (point)))
    (dolist (pair list)
      (let* ((alist (cdr pair))
             (open (cdr (assq 'open alist))))
        (insert "[" (if open "-" "+") "]"
                (car pair) "\n")
        (put-text-property prev (point) 'category (car pair))
        (put-text-property prev (1- (point)) 'mouse-face 'highlight)
        (when open
          (navi2ch-list-insert-board-names-subr (cdr (assq 'child alist))))
        (setq prev (point))))))

(defun navi2ch-list-toggle-open ()
  "$B%+%F%4%j$r3+$$$?$jJD$8$?$j$9$k!#(B"
  (interactive)
  (save-excursion
    (let (category)
      (beginning-of-line)
      (while (not (looking-at "^\\[[+-]\\]"))
        (forward-line -1))
      (setq category (get-text-property (point) 'category))
      (let* ((pair (assoc category navi2ch-list-category-list))
             (alist (cdr pair))
             (open (cdr (assq 'open alist)))
             (buffer-read-only nil))
        (delete-region (point) (+ 3 (point)))
        (insert "[" (if (not open) "-" "+") "]")
        (put-text-property (- (point) 3) (point) 'category (car pair))
        (forward-line 1)
        (if open
            (let ((begin (point)))
              (while (not (looking-at "^\\[[+-]\\]"))
                (forward-line 1))
              (delete-region begin (point)))
          (navi2ch-list-insert-board-names-subr (cdr (assq 'child alist))))
        (setcdr pair (navi2ch-put-alist 'open (not open) alist))))))

(defun navi2ch-list-select-current-board (&optional force)
  "$BHD$rA*$V(B"
  (interactive "P")
  (let ((board (get-text-property (point) 'board)))
    (if board
        (navi2ch-list-select-board board force)
      (message "can't select this line!"))))

(defun navi2ch-list-select-board (board &optional force)
  (when (get-buffer navi2ch-board-buffer-name)
    (delete-windows-on navi2ch-board-buffer-name))
  (dolist (x (navi2ch-article-buffer-list))
    (when x
      (delete-windows-on x)))
  (split-window-horizontally navi2ch-list-window-width)
  (other-window 1)
  (navi2ch-board-select-board board force))

(defun navi2ch-list-mode ()
  "\\{navi2ch-list-mode-map}"
  (interactive)
  (setq major-mode 'navi2ch-list-mode)
  (setq mode-name "Navi2ch List")
  (setq buffer-read-only t)
  (use-local-map navi2ch-list-mode-map)
  (run-hooks 'navi2ch-list-mode-hook)
  (force-mode-line-update))

(defun navi2ch-list ()
  (interactive)
  (if (get-buffer navi2ch-list-buffer-name)
      (switch-to-buffer navi2ch-list-buffer-name)
    (switch-to-buffer (get-buffer-create navi2ch-list-buffer-name))
    (navi2ch-list-mode)
    (save-excursion
      (navi2ch-list-sync nil t))))

(defun navi2ch-list-sync (&optional force first)
  (interactive "P")
  (save-excursion
    (let ((buffer-read-only nil)
          (navi2ch-net-force-update (or navi2ch-net-force-update
                                        force))
          (file (navi2ch-list-get-file-name))
          updated)
      (unless navi2ch-offline
        (setq updated (navi2ch-net-update-file
                       navi2ch-list-bbstable-url file nil
                       'navi2ch-list-make-board-txt)))
      (when (or first updated)
        (erase-buffer)
        (setq navi2ch-list-category-list
              (navi2ch-list-get-category-list file))
        (navi2ch-list-insert-board-names navi2ch-list-category-list)))))

(defun navi2ch-list-make-board-txt (str)
  "bbstable.html $B$+$i(B (navi2ch $BMQ$N(B) board.txt $B$r:n$k(B
`navi2ch-net-update-file' $B$N%O%s%I%i!#(B"
  (let ((coding-system-for-read 'binary)
        (coding-system-for-write 'binary))
    (with-temp-buffer
      (insert str)
      (let ((case-fold-search t)
            str2 start ignore)
        (goto-char (point-min))
        (while (re-search-forward
                "<\\([ab]\\)\\([^>]*\\)>\\([^<]+\\)</\\1>" nil t)
          (let ((tag (match-string 1))
                (href (match-string 2))
                (cont (match-string 3)))
            (setq str2
                  (concat str2
                          (if (string= tag "A")
                              (when (and start
                                         (not ignore))
                                (when (string-match
                                       "href=\\(.+/\\([^/]+\\)/\\)index2.html"
                                       href)
                                  (concat cont "\n"
                                          (match-string 1 href) "\n"
                                          (match-string 2 href) "\n")))
                            (unless start
                              (setq start t))
                            (setq ignore
                                  (member (decode-coding-string
                                           cont navi2ch-net-coding-system)
                                          navi2ch-list-ignore-category-list))
                            (when (not ignore)
                              (concat cont "\n\n\n")))))))
        str2))))

(defun navi2ch-list-mouse-select (e)
  (interactive "e")
  (beginning-of-line)
  (mouse-set-point e)
  (save-excursion
    (navi2ch-list-select-current-board)))

(defun navi2ch-list-goto-board ()
  (interactive)
  (let (alist board)
    (setq alist (mapcar
                 (lambda (x) (cons (cdr (assq 'id x)) x))
                 (navi2ch-list-get-board-name-list)))
    (save-window-excursion
      (setq board (cdr (assoc (completing-read "board name: "
                                               alist nil t)
                              alist))))
    (when board
      (navi2ch-board-save-info)
      (navi2ch-list-select-board board))))

(defun navi2ch-list-get-board-name-list ()
  (let (alist)
    (dolist (x navi2ch-list-category-list)
      (setq alist (append (cdr (assq 'child x)) alist)))
    alist))

(defun navi2ch-list-two-pain ()
  (interactive)
  (let* ((list-buf (get-buffer navi2ch-list-buffer-name))
         (board-buf (get-buffer navi2ch-board-buffer-name))
         (art-buf (navi2ch-article-current-buffer))
         (board-win (get-buffer-window (or board-buf "")))
         (art-win (get-buffer-window (or art-buf "")))
         buf)
    (when list-buf
      (delete-other-windows)
      (switch-to-buffer list-buf)
      (setq buf
            (cond ((and board-buf art-buf)
                   (cond ((and board-win art-win) board-buf)
                         (board-win art-buf)
                         (art-win board-buf)
                         (t board-buf)))
                  (board-buf board-buf)
                  (art-buf art-buf)))
      (when buf
        (split-window-horizontally navi2ch-list-window-width)
        (other-window 1)
        (switch-to-buffer buf))
      (other-window 1))))

(provide 'navi2ch-list)

;;; navi2ch-list.el ends here